<?php

namespace Hyiplab\Controllers\User;

use Hyiplab\BackOffice\Request;
use Hyiplab\Controllers\Controller;
use Hyiplab\Lib\FormProcessor;
use Hyiplab\Models\Form;
use Hyiplab\Models\Transaction;
use Hyiplab\Models\Withdrawal;
use Hyiplab\Models\WithdrawMethod;

class WithdrawController extends Controller
{
    public function index()
    {
        $this->pageTitle = __('Withdraw', HYIPLAB_PLUGIN_NAME);
        $methods         = WithdrawMethod::where('status', 1)->get();
        $this->view('user/withdraw/methods', compact('methods'));
    }

    public function insert()
    {
        $request = new Request();
        $request->validate([
            'method_code' => 'required',
            'amount'      => 'required|numeric'
        ]);

        $amount = sanitize_text_field($request->amount);
        $method = WithdrawMethod::where('id', sanitize_text_field($request->method_code))->where('status', 1)->first();

        $user = viserAuth()->user;

        if ($amount < $method->min_limit) {
            $notify[] = ['error', 'Your requested amount is smaller than minimum amount.'];
            viserBack($notify);
        }

        if ($amount > $method->max_limit) {
            $notify[] = ['error', 'Your requested amount is larger than maximum amount.'];
            viserBack($notify);
        }

        if ($amount > hyiplabBalance($user->ID, 'interest_wallet')) {
            $notify[] = ['error', 'You do not have sufficient balance for withdraw.'];
            viserBack($notify);
        }

        $charge      = $method->fixed_charge + ($amount * $method->percent_charge / 100);
        $afterCharge = $amount - $charge;
        $finalAmount = $afterCharge * $method->rate;

        $trx = vlTrx();
        $withdraw               = new Withdrawal();
        $withdraw->method_id    = $method->id;            // wallet method ID
        $withdraw->user_id      = $user->ID;
        $withdraw->amount       = $amount;
        $withdraw->currency     = $method->currency;
        $withdraw->rate         = $method->rate;
        $withdraw->charge       = $charge;
        $withdraw->final_amount = $finalAmount;
        $withdraw->after_charge = $afterCharge;
        $withdraw->trx          = $trx;
        $withdraw->created_at   = current_time('mysql');
        $withdraw->updated_at   = current_time('mysql');
        $withdraw->save();

        viserSession()->put('trx', $trx);
        viserRedirect(vlRoute('user.withdraw.preview'));
    }

    public function preview()
    {
        $this->pageTitle = __('Withdraw Preview', HYIPLAB_PLUGIN_NAME);
        $trx             = viserSession()->get('trx');
        $withdraw        = Withdrawal::where('trx', $trx)->where('status', 0)->orderBy('id', 'desc')->first();
        if (!$withdraw) {
            $notify[] = ['error', 'Withdraw not found'];
            setNotify($notify);
            viserRedirect(vlRoute('user.withdraw.preview'));
        }
        $method = WithdrawMethod::find($withdraw->method_id);
        $this->view('user/withdraw/preview', compact('withdraw', 'method'));
    }

    public function submit()
    {
        $request  = new Request();
        $trx      = viserSession()->get('trx');
        $withdraw = Withdrawal::where('trx', $trx)->where('status', 0)->orderBy('id', 'desc')->first();
        if (!$withdraw) {
            $notify[] = ['error', 'Withdraw not found'];
            setNotify($notify);
            viserRedirect(vlRoute('user.withdraw.index'));
        }

        $method        = WithdrawMethod::where('id', $withdraw->method_id)->first();
        $form          = Form::find($method->form_id);
        $formData      = json_decode(json_encode(maybe_unserialize($form->form_data)));
        $formProcessor = new FormProcessor();
        $userData      = $formProcessor->processFormData($request, $formData);

        $user = get_userdata($withdraw->user_id);

        if ($withdraw->amount > hyiplabBalance($user->ID, 'interest_wallet')) {
            $notify[] = ['error', 'Your request amount is larger then your current balance.'];
            setNotify($notify);
            viserRedirect(vlRoute('user.withdraw.index'));
        }

        $data['status']               = 2;
        $data['withdraw_information'] = maybe_serialize($userData);
        Withdrawal::where('id', $withdraw->id)->update($data);

        $afterBalance = hyiplabBalance($withdraw->user_id, 'interest_wallet') - $withdraw->amount;
        update_user_meta($withdraw->user_id, "hyiplab_interest_wallet", $afterBalance);

        $transaction               = new Transaction();
        $transaction->user_id      = $withdraw->user_id;
        $transaction->amount       = $withdraw->amount;
        $transaction->post_balance = $afterBalance;
        $transaction->charge       = $withdraw->charge;
        $transaction->trx_type     = '-';
        $transaction->details      = vlShowAmount($withdraw->final_amount) . ' ' . $withdraw->currency . __(' Withdraw Via ', HYIPLAB_PLUGIN_NAME) . $method->name;
        $transaction->trx          = $withdraw->trx;
        $transaction->wallet_type  = "interest_wallet";
        $transaction->remark       = 'withdraw';
        $transaction->created_at   = current_time('mysql');
        $transaction->save();

        vlNotify($user, 'WITHDRAW_REQUEST', [
            'method_name'     => $method->name,
            'method_currency' => $withdraw->currency,
            'method_amount'   => vlShowAmount($withdraw->final_amount),
            'amount'          => vlShowAmount($withdraw->amount),
            'charge'          => vlShowAmount($withdraw->charge),
            'rate'            => vlShowAmount($withdraw->rate),
            'trx'             => $withdraw->trx,
            'post_balance'    => vlShowAmount($afterBalance),
        ]);

        $notify[] = ['success', 'Withdraw request sent successfully'];
        setNotify($notify);
        viserRedirect(vlRoute('user.withdraw.history'));
    }

    public function history()
    {
        global $user_ID;
        $this->pageTitle = __('Withdraw History', HYIPLAB_PLUGIN_NAME);
        $withdraws       = Withdrawal::where('user_id', $user_ID)->where('status', '!=', 0)->orderBy('id', 'desc')->paginate(vlPaginate());
        $this->view('user/withdraw/history', compact('withdraws'));
    }
}
